#!/bin/bash

if [[ $EUID -eq 0 ]]; then
	IS_ROOT=true
else
	IS_ROOT=false
fi

IOB_DIR="/opt/iobroker"
CONTROLLER_DIR="$IOB_DIR/node_modules/iobroker.js-controller"

create_user_linux() {
	username="$1"
	id "$username" &> /dev/null;
	if [ $? -ne 0 ]; then
		# User does not exist
		if [ "$IS_ROOT" = true ]; then
			useradd -m -s /usr/sbin/nologin "$username"
		else
			sudo useradd -m -s /usr/sbin/nologin "$username"
		fi
		echo "User $username created"
	fi
	# Add the current non-root user to the iobroker group so he can access the iobroker dir
	if [ "$username" != "$USER" ] && [ "$IS_ROOT" = false ]; then
		sudo usermod -a -G $username $USER
	fi

	# Add the user to all groups we need and give him passwordless sudo privileges
	# Define which commands iobroker may execute as sudo without password
	declare -a iob_commands=(
		"shutdown -h now" "halt" "poweroff" "reboot"
		"systemctl start" "systemctl stop"
		"mount" "umount" "systemd-run"
		"apt-get" "apt" "dpkg" "make"
		"ping" "fping"
		"arp-scan"
		"setcap"
	)

	SUDOERS_CONTENT="$username ALL=(ALL) ALL\n"
	for cmd in "${iob_commands[@]}"; do
		# Test each command if and where it is installed
		cmd_bin=$(echo $cmd | cut -d ' ' -f1)
		cmd_path=$(which $cmd_bin 2> /dev/null)
		if [ $? -eq 0 ]; then
			# Then add the command to SUDOERS_CONTENT
			full_cmd=$(echo "$cmd" | sed -e "s|$cmd_bin|$cmd_path|")
			SUDOERS_CONTENT+="$username ALL=(ALL) NOPASSWD: $full_cmd\n"
		fi
	done

	# Additionally, define which iobroker-related commands may be executed by every user
	declare -a all_user_commands=(
		"systemctl start iobroker"
		"systemctl stop iobroker"
		"systemctl restart iobroker"
	)
	for cmd in "${all_user_commands[@]}"; do
		# Test each command if and where it is installed
		cmd_bin=$(echo $cmd | cut -d ' ' -f1)
		cmd_path=$(which $cmd_bin 2> /dev/null)
		if [ $? -eq 0 ]; then
			# Then add the command to SUDOERS_CONTENT
			full_cmd=$(echo "$cmd" | sed -e "s|$cmd_bin|$cmd_path|")
			SUDOERS_CONTENT+="ALL ALL=NOPASSWD: $full_cmd\n"
		fi
	done

	# Furthermore, allow all users to execute node iobroker.js as iobroker
	if [ "$username" != "$USER" ]; then
		cmd="node $CONTROLLER_DIR/iobroker.js"
		cmd_bin=$(echo $cmd | cut -d ' ' -f1)
		cmd_path=$(which $cmd_bin 2> /dev/null)
		if [ $? -eq 0 ]; then
			# Then add the command to SUDOERS_CONTENT
			full_cmd=$(echo "$cmd" | sed -e "s|$cmd_bin|$cmd_path|")
			SUDOERS_CONTENT+="ALL ALL=($username) NOPASSWD: $full_cmd\n"
		fi
	fi
	# TODO: ^ Can we reduce code repetition in these 3 blocks? ^

	SUDOERS_FILE="/etc/sudoers.d/iobroker"
	if [ "$IS_ROOT" = true ]; then
		echo -e "$SUDOERS_CONTENT" > ./temp_sudo_file
		visudo -c -q -f ./temp_sudo_file && \
			chown root:$ROOT_GROUP ./temp_sudo_file &&
			chmod 440 ./temp_sudo_file &&
			mv ./temp_sudo_file $SUDOERS_FILE &&
			echo "Created $SUDOERS_FILE"
	else
		echo -e "$SUDOERS_CONTENT" > ./temp_sudo_file
		sudo visudo -c -q -f ./temp_sudo_file && \
			sudo chown root:$ROOT_GROUP ./temp_sudo_file &&
			sudo chmod 440 ./temp_sudo_file &&
			sudo mv ./temp_sudo_file $SUDOERS_FILE &&
			echo "Created $SUDOERS_FILE"
	fi
	# Add the user to all groups if they exist
	declare -a groups=(
		bluetooth
		dialout
		gpio
		i2c
		tty
	)
	for grp in "${groups[@]}"; do
		if [ "$IS_ROOT" = true ]; then
			getent group $grp &> /dev/null && usermod -a -G $grp $username
		else
			getent group $grp &> /dev/null && sudo usermod -a -G $grp $username
		fi
	done
}

create_user_linux iobroker